﻿using UnityEngine;
using System.Collections.Generic;
using System.Linq;

public class CameraPlacement : MonoBehaviour
{
    public static void PlaceDefaultCamera(List<OCFBX> Gos)
    {
        var sortedGos = Gos.OrderByDescending(go => CountVertices(go.gameObject)).ToList();
        int subsetCount = Mathf.Max(1, sortedGos.Count / 5); // 20% объектов
        var selectedGos = sortedGos.Take(subsetCount).ToList();

        Bounds bounds = CalculateBounds(selectedGos);
        Camera newCamera=  new GameObject("DefaultCamera").AddComponent<Camera>();
        newCamera.transform.position = CalculateCameraPosition(bounds, newCamera);
        newCamera.transform.SetParent(FindObjectOfType<OCCamerasManager>().transform, false);
        newCamera.transform.LookAt(bounds.center);
        SetCameraFOV(newCamera);
    }

    private static int CountVertices(GameObject go)
    {
        MeshFilter meshFilter = go.GetComponent<MeshFilter>();
        return meshFilter != null ? meshFilter.sharedMesh.vertexCount : 0;
    }

    private static Bounds CalculateBounds(List<OCFBX> objects)

    {
        if (objects.Count == 0)
            return new Bounds();

        var bounds = new Bounds(objects[0].transform.position, Vector3.zero);
        foreach (var obj in objects)
        {
            bounds.Encapsulate(obj.GetComponent<Renderer>().bounds);
        }
        return bounds;
    }

    private static Vector3 CalculateCameraPosition(Bounds bounds, Camera camera)
    {
        float cameraDistance = 1.0f; // Начальное расстояние
        float maxDistance = 100.0f; // Максимальное расстояние
        float step = 1.0f; // Шаг изменения расстояния

        while (cameraDistance < maxDistance)
        {
            camera.transform.position = bounds.center - camera.transform.forward * cameraDistance;
            if (IsBoundsVisible(bounds, camera))
                break;

            cameraDistance += step;
        }

        return camera.transform.position;
    }

    private static bool IsBoundsVisible(Bounds bounds, Camera camera)
    {
        var planes = GeometryUtility.CalculateFrustumPlanes(camera);
        return GeometryUtility.TestPlanesAABB(planes, bounds);
    }

    private static void SetCameraFOV(Camera camera)
    {
        float focalLength = 50; // Фокусное расстояние в мм
        camera.fieldOfView = FocalLengthToFoV(focalLength, camera.sensorSize.y);
    }

    private static float FocalLengthToFoV(float focalLength, float sensorHeight)
    {
        return 2 * Mathf.Atan(sensorHeight / (2 * focalLength)) * Mathf.Rad2Deg;
    }
}