using UnityEngine;
using UnityEngine.SceneManagement;
using System.IO;
using System;
using System.Linq;
using System.Diagnostics;
using static UnityEngine.Rendering.DebugUI;

public static class Logger
{
    #region Fields and Properties

    public static int Version = 86;
    [SerializeField] public static OneClickSceneManager OCS;

    public static string unityVersion => Application.unityVersion;
    public static string currentScene => SceneManager.GetActiveScene().name;

    private static string _renderPipeLine;
    private static int _LogID = -1;

    public static string renderPipeline
    {
        get
        {
            if (_renderPipeLine == null)
            {
#if UNITY_PIPELINE_HDRP
                _renderPipeLine = "HDRP";
#endif
#if UNITY_PIPELINE_URP
                _renderPipeLine = "URP";
#endif
            }
            return _renderPipeLine;
        }
    } 
    private static string _pathLog="";
    public static string pathLog
    {
        get
        {
            if( String.IsNullOrEmpty(_pathLog))
            if (!Application.isEditor) 
            {
                 _pathLog = GetEnvironmentVariable("logPath");
            }
            else
            {
                _pathLog = Path.GetFullPath(Path.Combine(OCS.ProductPath,OCS.OCScene.OcName + ".log"));
            }
        
            return _pathLog;
        }
        set
        {
            SetEnvironmentVariable("logPath", value);
            _pathLog = value;
        }
    }
    private static string _VlogPath="";
    public static string VlogPath
    {
        get
        { 

            return _VlogPath;
        }
        set
        { 
            _VlogPath = value;
        }
    }

    public static int LogID
    {
        get
        {
            if (_LogID == -1) ;
          
            return _LogID;
        }
        set
        { 
            _LogID = value;
        }
    }

    #endregion

    #region Logging Methods

    public static void CheckVersion()
    {
        //if (File.Exists(logPath))
        //{
        //    File.Delete(logPath);
        //}
         
    }

    public static string RendersPath;
    public static string ImagePath;
    public static void Ini(OneClickSceneManager oc)
    {
#if UNITY_EDITOR
        CheckVersion();
        OCS = oc;
        if (pathLog == "") pathLog = Path.GetFullPath(Path.Combine(OCS.ProductPath,OCS.OCScene.OcName + ".log"));

        LogID = 0;

        VlogPath = Path.GetFullPath(Path.Combine(OCS.Path_Renders, OCS.OCScene.OcName + ".rfp"));
        string filePath = FindFilePath("Ocbackground0000.jpg");
        if (!string.IsNullOrEmpty(filePath))
        {
            UnityEngine.Debug.Log("File found: " + filePath);
        }
        ImagePath = filePath;

       
#else
         
       
       
#endif


#if UNITY_EDITOR
        Log("Logger is Initialized from Editor.");
#else
//string l= $"Logger is Initialized from Exe file: {Application.dataPath}\nJSON: {oc.JsonDirectory}\nLog: {logPath}\nLogID: {LogID}";
   //     Log(l);
         
#endif
    }
    public static string Status;
    public static void Log(string additionalInfo)
    {
#if UNITY_EDITOR
        UnityEngine.Debug.Log($"{additionalInfo}");
        if( OCS == null)  return;
        if (String.IsNullOrEmpty( OCS.ProductPath )) return; 

        if (String.IsNullOrEmpty(pathLog)) pathLog = Path.GetFullPath(Path.Combine(OCS.ProductPath,OCS.OCScene.OcName + ".log"));
      

        //if (logPath =="" || VlogPath =="" || OCS==null) return;

       
        
      
         SetEnvironmentVariable("LogID", LogID.ToString());

#endif
        string vers = "unknown";
        if (OCS != null)
            if (OCS.ConversionVersion != null) vers = OCS.ConversionVersion;

        string logContent = "[" + "LogID:" + LogID + "]\n " + $" [{DateTime.Now}]{vers}\n" +
                          additionalInfo.ToUpper() + "\n\n";



        LogID++;
        if (additionalInfo == null) additionalInfo = "NUll";
            if (File.Exists(pathLog))
        File.AppendAllText(pathLog, logContent);
        else
        {
            if (!Directory.Exists(Path.GetDirectoryName(pathLog)))
                Directory.CreateDirectory(Path.GetDirectoryName(pathLog));
            File.WriteAllText(pathLog, logContent);
        }
        //if (File.Exists(VlogPath)) File.Delete(VlogPath);
        //File.WriteAllText(VlogPath, logContent);
    } public static void Add(string additionalInfo)
    {
        UnityEngine.Debug.Log($"{additionalInfo}");
        if( OCS == null) return;
        if (OCS.ProductPath == "" ) return;
       OCS.OCScene.OcName = OCTools.CheckClone(OCS.gameObject.name);
#if UNITY_EDITOR
        if (pathLog == "") pathLog = Path.GetFullPath(Path.Combine(OCS.ProductPath,OCS.OCScene.OcName + ".log"));

#else
 
#endif
       
        if(File.Exists(pathLog))
        File.AppendAllText(pathLog,"\n"+ additionalInfo);
        else
        {
            if(Directory.Exists(Path.GetDirectoryName(pathLog))) 
            File.WriteAllText(pathLog, additionalInfo);
        }
        //if (File.Exists(VlogPath)) File.Delete(VlogPath);
        //File.WriteAllText(VlogPath, logContent);
    }

    public static void ClearOldLogs()
    {
        // Implementation needed
    }

#endregion

    #region Video Logging Methods

    static string ffmpegPath = @"C:\Program Files\renderfin\assets\ffmpeg.exe"; // Path to ffmpeg

    public static void VideoLog(string text)
    {
        string videologPath = Path.Combine(); // Path to text file
        int fontSize = CalculateFontSize(videologPath, 72, 24, 1000); // Calculating font size

        try
        {
            Process ffmpeg = new Process();
            ffmpeg.StartInfo.FileName = ffmpegPath;
            ffmpeg.StartInfo.Arguments = $"-loop 1 -i \"{ImagePath}\" -vf \"drawtext=fontfile='c\\:/OneClickUnityDefaultProjects/OCURP21+/Assets/OneClickImport/Textures/font.ttf':textfile='{videologPath.Replace("\\", "/")}':fontcolor=red:fontsize={fontSize}:x=(w-text_w)/2:y=(h-text_h)/2,scale=600:380\" -t 15 -pix_fmt yuv420p \"{Path.Combine(OCS.Path_Renders,OCS.OCScene.OcName + ".rfp")}\"";
            ffmpeg.StartInfo.UseShellExecute = true;
            ffmpeg.StartInfo.RedirectStandardOutput = true;
            ffmpeg.StartInfo.RedirectStandardError = true;
            ffmpeg.Start();
        }
        catch (Exception ex)
        {
            // Exception handling
        }
    }

    private static int CalculateFontSize(string filePath, int maxFontSize, int minFontSize, int maxCharacters)
    {
        string text = File.ReadAllText(filePath);
        int characterCount = text.Length;

        if (characterCount < maxCharacters)
        {
            // Linear interpolation
            float scale = (float)characterCount / maxCharacters;
            return (int)(minFontSize + (maxFontSize - minFontSize) * (1 - scale));
        }
        else
        {
            return minFontSize;
        }
    }

    #endregion

    #region Utility Methods

    public static void CheckAndTrimFile(string filePath)
    {
        // Implementation for checking and trimming file
    }

    
    public static string FindFilePath(string fileName)
    {
        string[] allFiles = Directory.GetFiles(Application.dataPath, fileName, SearchOption.AllDirectories);
        foreach (string file in allFiles)
        {
            if (Path.GetFileName(file) == fileName)
            {
                return file;
            }
        }

        UnityEngine.Debug.LogError("File not found: " + fileName);
        return null;
    }
    #endregion

    public static void SetEnvironmentVariable(string variableName, string value)
    {
#if UNITY_EDITOR
        try
        {
            string p = Path.Combine(Application.persistentDataPath, variableName + ".txt");
            if (!Directory.Exists(Path.GetDirectoryName((p))))
            {
                Directory.CreateDirectory(Path.GetDirectoryName((p)));
            }
            File.WriteAllText(p,value);
        }
        catch (Exception ex)
        {
            Log($"Error write FIle: {ex.Message}");
        }
#endif
    }
    public static string GetEnvironmentVariable(string variableName)
    {
#if !UNITY_EDITOR
        return File.ReadAllText(Path.Combine(Application.persistentDataPath, variableName + ".txt") );
#endif
        return "";
    }
}

