using UnityEngine;
using System.Diagnostics;
using System.IO;
using System;

public class OCImageConverter
{
    private static OCImageConverter _instance;
    public static OCImageConverter GetInstance(string ffmpegPath = default)
    {
            if (_instance == null || !string.IsNullOrEmpty(ffmpegPath) && _instance.ffmpegPath != ffmpegPath)
            {
                _instance = new OCImageConverter(ffmpegPath);
            }
            return _instance;
    }

    public string ffmpegPath;

    private OCImageConverter(string ffmpegPath = default)
    {

        if (string.IsNullOrEmpty(ffmpegPath) || !File.Exists(ffmpegPath))
        {
            this.ffmpegPath = FindFFmpegExecutable(); //     
        }
    }

    private string FindFFmpegExecutable()
    {
        return Path.Combine(
            Environment.GetFolderPath(Environment.SpecialFolder.ProgramFiles),
                "renderfin", "assets", "ffmpeg.exe");
    }

    //       ffmpeg.
    public void ConvertImageToPreview(string imagePath, int width, int height, string outputPath)
    {
        if (!File.Exists(ffmpegPath))
        {
            Logger.Log("Path to ffmpeg.exe is not set or incorrect.");
            return;
        }

        if (!File.Exists(imagePath))
        {
            UnityEngine.Debug.LogError("Input file does not exist: " + imagePath);
            return;
        }

        var imageResolution = GetFileResolution(imagePath);
        string arguments = $"-i \"{imagePath}\" -vf scale={width}:{height} \"{outputPath}\"";

        ProcessStartInfo startInfo = new ProcessStartInfo(ffmpegPath, arguments)
        {
            CreateNoWindow = true,
            UseShellExecute = false,
            RedirectStandardOutput = true,
            RedirectStandardError = true
        };

        using (Process process = Process.Start(startInfo))
        {
            using (StreamReader reader = process.StandardError)
            {
                string result = reader.ReadToEnd();
                UnityEngine.Debug.Log(result);
            }

            process.WaitForExit();
        }

        if (File.Exists(outputPath))
        {
            UnityEngine.Debug.Log("Image converted successfully to: " + outputPath);
        }
        else
        {
            UnityEngine.Debug.LogError("Image conversion failed.");
        }
    }

    //        .
    public (int width, int height) GetFileResolution(string filePath)
    {
        if (!File.Exists(filePath))
        {
            Logger.Log("  : " + filePath);
            return (0, 0);
        }

        //       .
        Texture2D texture = new Texture2D(2, 2);
        byte[] imageBytes = File.ReadAllBytes(filePath);

        // LoadImage       .
        //   false   ,    ,        .
        if (texture.LoadImage(imageBytes, false))
        {
            return (texture.width, texture.height);
        }
        else
        {
            Logger.Log("     : " + filePath);
            return (0, 0);
        }
    }
}

