#if UNITY_EDITOR
using UnityEngine;
using UnityEngine.Rendering;
using UnityEditor; 
using static OCImportProjects;
using System.IO;

public class OCImportWindow : EditorWindow
{
    private Texture2D textureToShow;
    void OnGUI()
    {
        //EditorGUILayout.BeginHorizontal();
        //EditorGUILayout.BeginVertical();
        if (textureToShow == null) LoadTextureFromFile();
        GUILayout.Label("Select Available Scene to import:", EditorStyles.boldLabel);

        //if (textureToShow != null)
        //{
        //    Rect textureRect = new Rect(100, 10, 70, 70);
        //    GUI.DrawTexture(textureRect, textureToShow);
        //}
        //else
        //{
        //    GUILayout.Label("Texture not loaded");
        //}
        //EditorGUILayout.EndVertical();


        if (OCScenes.Count > 0)
            selectedIndex = EditorGUILayout.Popup(selectedIndex, OCScenes.ConvertAll(s => s[0]).ToArray());


        if (GUILayout.Button("ImportProjectToScene"))
        {
            ImportProjectToScene(selectedIndex);
        } 
        //if (GUILayout.Button(@$"Conveyour Pipeline:[{OCScenes.Count}]"))
        //{
        //    ImportProjectToScene(selectedIndex);
        //}
        //EditorGUILayout.EndHorizontal();
    }
    private void ConveyourPipeLine(int _selectedSceneIndex)
    {

    }
    private void ImportProjectToScene(int _selectedSceneIndex)
    {

        ClearScene();
        DisableComponentsInScene();
        if (OCScenes.Count == 0) 
        {
            Logger.Log("OCScenes Count: " + OCScenes.Count);
            return;
        };

            //   GameObject  OneClickSceneManager
            var newManagerObject = new GameObject("OneClickSceneManager");
        OneClickSceneManager oc = newManagerObject.AddComponent<OneClickSceneManager>();

        OCScene ocs =  oc.gameObject.AddComponent<OCScene>();
        ocs.OcName = OCImportProjects.OCScenes[_selectedSceneIndex][0];
        ocs.ProductDirectory= OCImportProjects.OCScenes[_selectedSceneIndex][1];
        oc.OCScene = ocs;
        ocs.ocsm = oc;
        oc.isBatchRender = false;
        oc.ImportScene3dsmax();
    }

    private int selectedSceneIndex ;

    [InitializeOnLoadMethod]
    static void OnProjectLoad()
    {
        EditorApplication.delayCall += OpenWindowOnStart;
    }

    static void OpenWindowOnStart()
    {
            ImportJsonFiles();

        //if (!EditorPrefs.GetBool("IsWindowOpened", false))
        //{
            OpenWindow();
            EditorPrefs.SetBool("IsWindowOpened", true);
        //}
    }

    [MenuItem("Window/One Click Importer")]
    public static void OpenWindow()
    {
        var window = GetWindow<OCImportWindow>("One Click Importer");
        window.Show();
    }

    int selectedIndex;
   

    void OnDestroy()
    {
        EditorPrefs.SetBool("IsWindowOpened", false);
    }

  
    public static void DisableComponentsInScene()
    {
         
        foreach (var camera in Object.FindObjectsOfType<Camera>())
        {
            camera.enabled = false;
        }

         
        foreach (var volume in Object.FindObjectsOfType<Volume>())
        {
            volume.enabled = false;
        }

         
        foreach (var probe in Object.FindObjectsOfType<ReflectionProbe>())
        {
            probe.enabled = false;
        }
         //    
        foreach (var light in Object.FindObjectsOfType<Light>())
        {
            light.enabled = false;
        }
    }
    private void ClearScene()
    {
         
        var existingManager = FindObjectOfType<OneClickSceneManager>();
        if (existingManager != null)
        {
            DestroyImmediate(existingManager.gameObject);
        }
        OCCamerasManager existingManagerCameras = FindObjectOfType<OCCamerasManager>();
        if (existingManager != null)
        {
            DestroyImmediate(existingManagerCameras.gameObject);
        }
        OCLightingManager existingManagerLighting = FindObjectOfType<OCLightingManager>();
        if (existingManager != null)
        {
            DestroyImmediate(existingManagerLighting.gameObject);
        }
        OCVolumeManager existingManagerVolume = FindObjectOfType<OCVolumeManager>();
        if (existingManager != null)
        {
            DestroyImmediate(existingManagerVolume.gameObject);
        }OCPrefab existingManagerPrefab = FindObjectOfType<OCPrefab>();
        if (existingManager != null)
        {
            DestroyImmediate(existingManagerVolume.gameObject);
        }
    }
    private string filePath => Application.dataPath+"\\OneClickImport\\Textures\\Ocbackground0000.JPG";

    private void LoadTextureFromFile()
    {
        if (File.Exists(filePath))
        {
            byte[] fileData = File.ReadAllBytes( filePath);
            textureToShow = new Texture2D(2, 2);
            textureToShow.LoadImage(fileData);
        }
        else
        {
            Debug.LogError("File not found at: " + filePath);
        }
    }
}

#endif

