
using System.Collections;
using System.Collections.Generic;
using System.Drawing;
using System.Xml.Linq;
#if UNITY_EDITOR
using UnityEditor;
using UnityEditorInternal;
#endif
using UnityEngine;
using UnityEngine.Experimental.GlobalIllumination;

#if UNITY_PIPELINE_HDRP
using UnityEngine.Rendering.HighDefinition;
#endif
using UnityEngine.UIElements;
using UnityEngine.Events;
using System.Linq;
using LightType = UnityEngine.LightType;
[ExecuteInEditMode]

public class OCLightingManager : MonoBehaviour
{

    public static OCLightingManager instance;
   [SerializeField] public List<OCVrayLight> Lights = new List<OCVrayLight>();
    void Awake()
    {
        
        if (instance == null)
        {
            instance = this;
        }
        else
        {
            DestroyImmediate(gameObject);
        }
        name = "OCLightingManager";
        transform.SetAsFirstSibling();
    }
    OneClickSceneManager _ocsm;
    OneClickSceneManager ocsm
    {
        get 
        {
             if (_ocsm == null) _ocsm = FindObjectOfType<OneClickSceneManager>();
            return _ocsm;
        }
    }

#if UNITY_PIPELINE_HDRP
HDAdditionalLightData lightData ;
#endif
   
    public void ChangeMults(float AreaLightsMult , float SpotLightsMult , float DirectionalLightsMult , float PointLightsMult )
    {
        if(Lights.Count>0)

#if UNITY_PIPELINE_HDRP
            foreach (var item in Lights)
        {
            if(item.lightData.type == HDLightType.Area) 
                    item.lightData.intensity = item.multiplier*AreaLightsMult;       
            if(item.lightData.type == HDLightType.Spot)
                    item.lightData.intensity = item.multiplier*SpotLightsMult;
            if(item.lightData.type == HDLightType.Directional)
                    item.lightData.intensity = item.multiplier*DirectionalLightsMult;
            if(item.lightData.type == HDLightType.Point)
                    item.lightData.intensity = item.multiplier*PointLightsMult;
         }
#else
  foreach (var item in Lights)
        {
            if((int)item.l.type == 3) 
                    item.l.intensity = item.multiplier*AreaLightsMult;       
            if(item.l.type == LightType.Spot)
                    item.l.intensity = item.multiplier*SpotLightsMult;
            if(item.l.type == LightType.Directional)
                    item.l.intensity = item.multiplier*DirectionalLightsMult;
            if(item.l.type == LightType.Point)
                    item.l.intensity = item.multiplier*PointLightsMult;
         }
#endif

    }
#if UNITY_EDITOR
    public void AddLight(OCObject obj, Quaternion rot, Vector3 targetPos)
    {
         
        

            OCVrayLight O = new GameObject().AddComponent<OCVrayLight>();

        Lights.Add(O);

        O.gameObject.transform.SetParent(transform);
        O.gameObject.name = obj.ocname;
        O.transform.position = targetPos;
        O.transform.rotation = rot;
        O.type = obj.type;
        Light light = O.gameObject.AddComponent<Light>();
        O.l = light;
        O.l.renderMode = LightRenderMode.ForcePixel;
#if UNITY_PIPELINE_HDRP
        lightData = O.gameObject.GetComponent<HDAdditionalLightData>();
        if (lightData == null)         
            lightData = O.gameObject.AddComponent<HDAdditionalLightData>(); 
#endif

        if (obj.octype == "VRayLight" && O.type == 2) CreateVrayPointLight(O, obj, rot, targetPos);//point light

        if (obj.octype == "VRaySun") CreateVraySun(O, obj, rot, targetPos);

        if (obj.octype == "VRayLight" && O.type == 0) CreateVrayPlaneLight(O, obj, rot, targetPos);

        if (obj.octype == "VRayLight" && O.type == 1) CreateVrayDomeLight(O, obj, rot, targetPos);

        if (obj.octype == "VRayLight" && O.type == 4) CreateVrayDiscLight(O, obj, rot, targetPos);// || obj.shape==2)//disc light
     
        if (obj.octype == "CoronaLight" && obj.shape == 0) CreateCoronaPointLight(O, obj, rot, targetPos);//point light

        if (obj.octype == "CoronaSun") CreateCoronaSun(O, obj, rot, targetPos);

        if (obj.octype == "CoronaLight" && obj.shape == 1) CreateCoronaPlaneLight(O, obj, rot, targetPos);

        if (obj.octype == "CoronaLight" && obj.shape== 2) CreateCoronaDiscLight(O, obj, rot, targetPos);// || obj.shape==2)//disc light

        if (obj.octype == "CoronaLight" && obj.shape== 3) CreateCoronaCylinder(O, obj, rot, targetPos);// || obj.shape==2)//disc light

        light.enabled = (obj.enabled || obj.on);
#if UNITY_EDITOR
        light.lightmapBakeType = LightmapBakeType.Baked;
        OCFBX o = ocsm.ocfbxObjects.Find(X=>X.idx == obj.idx);
          light.areaSize = new Vector2(2*Mathf.Abs(obj.nodescale[0]) * obj.size0*ocsm.Units, 2*Mathf.Abs(obj.nodescale[1]) * obj.size1 * ocsm.Units);
#endif
        SortObjects();
    }

    #region VRAY
    void CreateVraySun(OCVrayLight O, OCObject obj, Quaternion rot, Vector3 TargetPos)
    {
        O.multiplier = O.l.intensity = Mathf.Clamp( obj.intensity_multiplier*5,0.1f,10);
        O.l.gameObject.name = "[Directional]" + " " + O.l.gameObject.name;

        O.l.type = UnityEngine.LightType.Directional;
        O.l.color = new UnityEngine.Color(obj.filter_Color[0] / 255f, obj.filter_Color[1] / 255f, obj.filter_Color[2] / 255f);
        O.l.color = new UnityEngine.Color(1, 0.9f, 0.6f);
        O.l.shadows = LightShadows.Soft;
 
        UnityEngine.RenderSettings.sun = O.l;
#if UNITY_PIPELINE_HDRP
        //    lightData.intensity =100* O.multiplier;// 100000 *obj.intensity_multiplier;
        lightData.intensity = Mathf.Clamp(100* O.multiplier, 0, 40000);
        O.multiplier = O.l.intensity = Mathf.Clamp(100 *obj.intensity_multiplier,0,40000);
        lightData.rayTraceContactShadow = true;
            lightData.SetShadowResolutionLevel(3);
            lightData.EnableShadows(true); 
            lightData.SetShadowResolution(2048);
#endif
    }
    void CreateVrayPointLight(OCVrayLight O, OCObject obj, Quaternion rot, Vector3 TargetPos)
    {
        O.multiplier = O.l.intensity = Normalizeligh(obj.multiplier/2);

        O.l.gameObject.name = "[Point]" + " " + O.l.gameObject.name;
        O.l.lightmapBakeType = LightmapBakeType.Baked;
        O.l.type = UnityEngine.LightType.Point;
        O.l.color = new UnityEngine.Color(obj.color[0] / 255f, obj.color[1] / 255f, obj.color[2] / 255f);
        O.l.shadowRadius = 0.33f;
 
        O.l.shadows = LightShadows.Soft;



#if UNITY_PIPELINE_HDRP
        lightData.intensity =  15* O.multiplier;// 1000 * obj.multiplier;
        lightData.intensity = Mathf.Clamp(lightData.intensity, 0, 40000);

            lightData.rayTraceContactShadow = true;
            lightData.SetShadowResolutionLevel(3);
            lightData.EnableShadows(true); 
            lightData.range=2; 
            lightData.SetShadowResolution(2048);
            lightData.lightmapBakeType = LightmapBakeType.Mixed;
#endif
    }
    void CreateVrayDomeLight(OCVrayLight O, OCObject obj, Quaternion rot, Vector3 TargetPos)
    {
        O.l.type = UnityEngine.LightType.Point;
        O.l.color = new UnityEngine.Color(obj.color[0] / 255f, obj.color[1] / 255f, obj.color[2] / 255f);
        O.multiplier = 0;
        O.l.enabled= false;
        if(obj.texmap!=null)
        {
            FindObjectOfType<OCVolumeManager>().EnvMap = FindObjectOfType<OneClickSceneManager>().Convert2DTextureToCubeMap( obj.texmap);
        }

#if UNITY_PIPELINE_HDRP
        lightData.intensity = 100 * O.multiplier;// 1000 * obj.multiplier;
        lightData.intensity = Mathf.Clamp(lightData.intensity, 0, 40000);

            lightData.rayTraceContactShadow = true;
            lightData.SetShadowResolutionLevel(3);
            lightData.EnableShadows(true); 
            lightData.range=2; 
            lightData.SetShadowResolution(2048);
            lightData.lightmapBakeType = LightmapBakeType.Mixed;
#endif
    }
    void CreateVrayPlaneLight(OCVrayLight O, OCObject obj, Quaternion rot, Vector3 TargetPos)
    {
        O.multiplier = O.l.intensity = Mathf.Clamp( Normalizeligh(obj.multiplier ), 0, 1);// *obj.sizeLength*obj.sizeWidth*u.Units;


        O.l.gameObject.name = "[Area]" + " " + O.l.gameObject.name;

        O.l.type = (LightType)3;
        O.l.color = new UnityEngine.Color(obj.color[0] / 255f, obj.color[1] / 255f, obj.color[2] / 255f);
         
        O.l.shadows = LightShadows.Soft;
        O.l.transform.Rotate(Vector3.right, 180);
         
        O.l.lightmapBakeType = LightmapBakeType.Mixed;
      //  O.l.areaSize = new Vector2(obj.sizeLength * ocsm.Units, obj.sizeWidth * ocsm.Units);
        //if(obj.invisible==false) O.l.enabled = false;
        CreateMeshLight(PrimitiveType.Cube, O, obj);
        //O.l.bounceIntensity = 10;
        O.l.shadowBias = 0.1f;
#if UNITY_PIPELINE_HDRP

        lightData.intensity = 1500* O.multiplier* obj.sizeLength * obj.sizeWidth;// obj.multiplier * 1500 * obj.sizeLength * obj.sizeWidth;//*u.Units;
        lightData.intensity = Mathf.Clamp(lightData.intensity, 0, 40000);
        lightData.lightmapBakeType = LightmapBakeType.Mixed;
        lightData.SetAreaLightSize(new Vector3(2 * Mathf.Abs(obj.nodescale[0]) * obj.size0 * ocsm.Units,0.01f, 2 * Mathf.Abs(obj.nodescale[1]) * obj.size1 * ocsm.Units));
#endif
    }
    void CreateVrayDiscLight(OCVrayLight O, OCObject obj, Quaternion rot, Vector3 TargetPos)
    {
        O.multiplier = O.l.intensity = Normalizeligh( 0.66f*obj.multiplier * 0.0001f);

        O.l.gameObject.name = "[Spot]" + " " + O.l.gameObject.name;


        O.l.transform.Rotate(Vector3.right, 180);
        O.l.type = UnityEngine.LightType.Spot;
        O.l.spotAngle = 110 - (60 * obj.lightDistribution); 
        O.l.color = new UnityEngine.Color(obj.color[0] / 255f, obj.color[1] / 255f, obj.color[2] / 255f);
     
        O.l.shadows = LightShadows.Soft;
        CreateMeshLight(PrimitiveType.Cylinder, O, obj);

#if UNITY_PIPELINE_HDRP
        lightData.intensity =30 * O.multiplier;// obj.multiplier * 1500 * obj.sizeLength * obj.sizeWidth;//*u.Units;
        lightData.intensity = Mathf.Clamp(lightData.intensity, 0, 40000);

        lightData.lightmapBakeType = LightmapBakeType.Mixed;
        lightData.SetAreaLightSize( new Vector3(2 * Mathf.Abs(obj.nodescale[0]) * obj.size0 * ocsm.Units,0.01f, 2 * Mathf.Abs(obj.nodescale[1]) * obj.size1 * ocsm.Units));


#endif

    }
    #endregion
    #region Corona
    void CreateCoronaSun(OCVrayLight O, OCObject obj, Quaternion rot, Vector3 TargetPos)
    {
        O.l.gameObject.name = "[Directional]" + " " + O.l.gameObject.name;
        O.l.type = UnityEngine.LightType.Directional;
        O.l.color = new UnityEngine.Color(obj.colorDirect[0] / 255f, obj.colorDirect[1] / 255f, obj.colorDirect[2] / 255f);
        O.multiplier = O.l.intensity =  obj.intensity;
        O.l.shadows = LightShadows.Soft; 

        //.transform.Rotate(Vector3.right, 180);
        UnityEngine.RenderSettings.sun = O.l;

#if UNITY_PIPELINE_HDRP
            lightData.intensity = obj.multiplier=1200;
            lightData.rayTraceContactShadow = true;
            lightData.SetShadowResolutionLevel(3);
            lightData.EnableShadows(true); 
            lightData.SetShadowResolution(2048);
#endif
    }
    void CreateCoronaPointLight(OCVrayLight O, OCObject obj, Quaternion rot, Vector3 TargetPos)
    {
        O.l.gameObject.name = "[Point]" + " " + O.l.gameObject.name;
        O.l.type = UnityEngine.LightType.Point;
        O.l.color = new UnityEngine.Color(obj.color[0] / 255f, obj.color[1] / 255f, obj.color[2] / 255f);
        O.multiplier = O.l.intensity = 1.5f*obj.intensity; 
        O.l.shadows = LightShadows.Soft;



#if UNITY_PIPELINE_HDRP
            lightData.intensity = obj.multiplier=1000;
            lightData.rayTraceContactShadow = true;
            lightData.SetShadowResolutionLevel(3);
            lightData.EnableShadows(true); 
            lightData.range=2; 
            lightData.SetShadowResolution(2048);
            lightData.lightmapBakeType = LightmapBakeType.Mixed;
#else
        O.multiplier = O.l.intensity = 100;
#endif
    }  void CreateCoronaCylinder(OCVrayLight O, OCObject obj, Quaternion rot, Vector3 TargetPos)
    {
        O.l.gameObject.name = "[Tube]" + " " + O.l.gameObject.name;
        O.l.type = UnityEngine.LightType.Point;
        O.l.color = new UnityEngine.Color(obj.color[0] / 255f, obj.color[1] / 255f, obj.color[2] / 255f);
        O.multiplier = O.l.intensity = 1.5f*obj.intensity; 
        O.l.shadows = LightShadows.Soft;



#if UNITY_PIPELINE_HDRP
        O.l.type = (LightType)3;

        lightData.lightmapBakeType = LightmapBakeType.Realtime;
#if !UNITY_2023_1_OR_NEWER
        lightData.areaLightShape = AreaLightShape.Tube;
#endif
        lightData.shapeWidth= obj.height * ocsm.Units;
        lightData.intensity = obj.multiplier=1000;
            lightData.rayTraceContactShadow = true;
            lightData.SetShadowResolutionLevel(3);
            lightData.EnableShadows(true); 
            lightData.range=2; 
            lightData.SetShadowResolution(2048);
#else
        O.multiplier = O.l.intensity = 100;
#endif
    }
    void CreateCoronaPlaneLight(OCVrayLight O, OCObject obj, Quaternion rot, Vector3 TargetPos)
    {
        O.l.gameObject.name = "[Area]" + " " + O.l.gameObject.name;
        O.l.type = (LightType)3;
        O.l.color = new UnityEngine.Color(obj.color[0] / 255f, obj.color[1] / 255f, obj.color[2] / 255f);
        O.multiplier = O.l.intensity =  obj.intensity; 
        O.l.shadows = LightShadows.Soft;
        O.l.transform.Rotate(Vector3.right, 180);
         
        O.l.lightmapBakeType = LightmapBakeType.Mixed;
        O.l.areaSize = new Vector2(obj.height * ocsm.Units, obj.width * ocsm.Units);
        O.l.bounceIntensity = 1;




#if UNITY_PIPELINE_HDRP
        lightData.shapeHeight = obj.height * ocsm.Units;
        lightData.shapeWidth= obj.width* ocsm.Units;
        lightData.intensity = obj.intensity;// * 1500 * obj.sizeLength * obj.sizeWidth;//*u.Units;
        lightData.lightmapBakeType = LightmapBakeType.Mixed; 
#else
        O.multiplier = O.l.intensity = 100;
#endif
    }
    void CreateCoronaDiscLight(OCVrayLight O, OCObject obj, Quaternion rot, Vector3 TargetPos)
    {
        O.l.gameObject.name = "[Spot]" + " " + O.l.gameObject.name;
        O.l.transform.Rotate(Vector3.right, 180);
        O.l.type = UnityEngine.LightType.Spot;
        O.l.innerSpotAngle =5 ;
        O.l.spotAngle = 110 - (60 * obj.directionality);
        O.l.color = new UnityEngine.Color(obj.color[0] / 255f, obj.color[1] / 255f, obj.color[2] / 255f);
        O.multiplier = O.l.intensity = obj.intensity ; 
        O.l.bounceIntensity = 1;
        O.l.shadows = LightShadows.Soft;
        O.l.shadowNearPlane = 10;
        CreateMeshLight(PrimitiveType.Sphere, O, obj);

#if UNITY_PIPELINE_HDRP
        lightData.intensity = obj.intensity=2000;// * 1500 * obj.sizeLength * obj.sizeWidth;//*u.Units;
        lightData.lightmapBakeType = LightmapBakeType.Mixed;
        lightData.SetAreaLightSize(new Vector2(obj.sizeLength * ocsm.Units, obj.sizeWidth * ocsm.Units));
#else
        O.multiplier = O.l.intensity = 100;
#endif

    }
    #endregion

   void CreateMeshLight(PrimitiveType pt,OCVrayLight O,OCObject obj)
    {

        if (obj.invisible == true) return;

        GameObject meshLight = GameObject.CreatePrimitive(pt);
        meshLight.transform.SetParent(O.transform);
        meshLight.transform.localPosition = Vector3.zero;
        meshLight.transform.localScale = new Vector3(2 * Mathf.Abs(obj.nodescale[0]) * obj.size0 * ocsm.Units,0.01f, 2 * Mathf.Abs(obj.nodescale[1]) * obj.size1 * ocsm.Units);


        if (obj.octype == "CoronaLight")//Corona Light Radius for spot
            meshLight.transform.localScale =2* new Vector3(obj.width * ocsm.Units, obj.width * ocsm.Units, obj.width * ocsm.Units);

        meshLight.transform.rotation = O.transform.rotation;
        meshLight.transform.Rotate(Vector3.right, 90);
        MeshRenderer m = meshLight.GetComponent<MeshRenderer>();
        m.sharedMaterial = new Material(m.sharedMaterial);
        m.gameObject.isStatic = false;
        
        m.shadowCastingMode = UnityEngine.Rendering.ShadowCastingMode.Off;
        m.scaleInLightmap = 0;
        m.sharedMaterial.SetColor("_EmissiveColor", O.l.color * 8.3f);
        m.sharedMaterial.SetColor("_EmissionColor", O.l.color * 8.3f);
        m.sharedMaterial.SetFloat("_EmissiveExposureWeight", 0.5f);
        m.sharedMaterial.EnableKeyword("_EMISSION");
        m.sharedMaterial.globalIlluminationFlags = MaterialGlobalIlluminationFlags.RealtimeEmissive;
        m.sharedMaterial.SetFloat("_EmissiveMult", O.multiplier * 200000);
        m.receiveGI = ReceiveGI.LightProbes;
        meshLight.isStatic = true;
        if (pt == PrimitiveType.Cylinder)
        {
            if(obj.octype== "VRayLight")
            {
                meshLight.transform.localScale = new Vector3(obj.size0 * ocsm.Units, 0.02f, 0.02f);
            }
            else
            meshLight.transform.localScale = new Vector3(obj.sizeLength * ocsm.Units, 0.01f, 0.01f);
        }
#if UNITY_PIPELINE_HDRP
     
        m.sharedMaterial.SetColor("_EmissiveColor", O.l.color * 3.3f);
        m.sharedMaterial.SetFloat("_EmissiveExposureWeight", 0.5f);
        m.sharedMaterial.EnableKeyword("_EMISSION");
        if (obj.octype == "CoronaLight")
        {
            m.sharedMaterial.SetColor("_EmissiveColor", UnityEngine.Color.white* 20.3f);
            m.sharedMaterial.SetColor("_EmissionColor", UnityEngine.Color.white * 20.3f);
            m.sharedMaterial.SetFloat("_EmissiveMult",  20000);

        }
        m.sharedMaterial.globalIlluminationFlags = MaterialGlobalIlluminationFlags.AnyEmissive;
#endif
         

    }
    public void SortObjects()
    {
        List<Transform> childObjects = new List<Transform>();
        foreach (Transform child in transform)
        {
            childObjects.Add(child);
        }

        //    
        childObjects.Sort((a, b) => string.Compare(a.name, b.name));
        for (int i = 0; i < childObjects.Count; i++)
        {
            childObjects[i].SetSiblingIndex(i);
        }
    }
#endif
    public float Normalizeligh(float val)
    {

#if !UNITY_PIPELINE_HDRP
        if (val > 10 && val <= 100) return val / 10;
        if (val > 100 && val <= 1000) return val / 100;
        if (val > 1000)
        {
            val /= 1000;
            val= Normalizeligh(val);
            return  Mathf.Clamp( val,0.3f,100);
        }
#endif
        return val;

    } 
}

 