using System; 
using System.Collections.Generic;
using System.IO;
using UnityEngine; 

public class OCRenderManager : OCBase
{
    public List<int> RenderBuffer;
    public OCScene scene; 
    public string BasePath
    {
        get
        {
            if (scene.ocsm.isBatchRender)
              return $"C:\\Users\\{Environment.UserName}\\Documents\\oc\\output\\";
              return scene.ocsm.ProductPath;
        }
    }
    private string _OutputProductFolder;
    public string OutputProductFolder
    {
        get
        { 
                if (!scene.ocsm.isBatchRender)
                    _OutputProductFolder = BasePath;
                else
                    _OutputProductFolder = BasePath + $@"{ZipName}[Unity]\{scene.OcName}{scene.ocsm.ConversionVersion}";

           // if (scene.ocsm.isBatchRender) CheckFolderForScene();

           // if (scene.ocsm.isBatchRender) GenerateFoldersForScene(_OutputProductFolder);
             
            return _OutputProductFolder;

        } 
    }
    public string ZipName;
    private string CheckFolderForScene()
    {
        foreach (var file in Directory.GetFiles(BasePath, "*.max", SearchOption.AllDirectories))
        {
            if (Path.GetFileNameWithoutExtension(file).Equals(scene.OcName, StringComparison.OrdinalIgnoreCase))
            {
                string findedScene = "";
                int k = file.IndexOf(scene.OcName);
                if (k > 0)
                {
                    findedScene = file.Substring(k); 

                    scene.ZipName = ZipName =OCTools.GetFirstFolder(file.Substring(BasePath.Length));

                    return findedScene;
                }
            }
        }

        return "";
    }
    public string GetFirstNCharacters(string str, int n)
    {
        if (string.IsNullOrEmpty(str) || n <= 0)
        {
            return ""; //   ,      n 
        }

        if (n > str.Length)
        {
            return str; //   ,  n   
        }

        return str.Substring(0, n); //   n  
    }
    public  string meshes_folder=> Path.GetFullPath(Path.Combine(OutputProductFolder, "..", "meshes"));
    public  string textures_folder => Path.GetFullPath(Path.Combine(OutputProductFolder, "..", "textures"));
    public string materials_folder => Path.GetFullPath(Path.Combine(OutputProductFolder, "..", "materials"));
    public string scenes_folder => OutputProductFolder + $"\\Scenes\\";
    public string builds_folder => OutputProductFolder + $"\\Builds\\";

    public string base_folder = @"C:\Users\user\Documents\oc\output";
    public string renders_folder => Path.GetFullPath(Path.Combine(OutputProductFolder, ".."));

    public void GenerateFoldersForScene(string res)
    {
        List <string> AllFolders = new List<string>();

        AllFolders.Add(base_folder);
        //AllFolders.Add(res);
        AllFolders.Add(meshes_folder);
        AllFolders.Add(textures_folder);
        AllFolders.Add(materials_folder);
        AllFolders.Add(scenes_folder);
        AllFolders.Add(builds_folder);
        AllFolders.Add(renders_folder);
        AllFolders.Add(meshes_folder);

        foreach (var item in AllFolders)
        if (!Directory.Exists(item))
        {
            Directory.CreateDirectory(item);
        }
    }
    public void CopyResources()
    {
        CopyDirectory(scene.ocsm.Path_Textures, textures_folder);

        CopyDirectory(scene.ocsm.Path_Meshes, meshes_folder);
        CopyDirectory(scene.ocsm.Path_Materials, materials_folder);
    }
    public void CopyDirectory(string sourceDir,string destinationDir)
    {
        {
              //         
            foreach (string file in Directory.GetFiles(sourceDir))
            {
                string destFile = Path.Combine(destinationDir, Path.GetFileName(file));
                File.Copy(file, destFile, true); // true    
            }

            //    
            foreach (string subDir in Directory.GetDirectories(sourceDir))
            {
                string destSubDir = Path.Combine(destinationDir, Path.GetFileName(subDir));
                CopyDirectory(subDir, destSubDir); //    
            }
        }
    }
    public string OCBackGroundSequence = @"C:\OneClickUnityDefaultProjects\OCURP21+\Assets\OneClickImport\Textures\Ocbackground0000.jpg";
    public void RenderPreviewFakeFile()
    {
        if(String.IsNullOrEmpty( OutputProductFolder)) return;
        FindObjectOfType<OCScreenshotTaker>(). CreateVideo(OCBackGroundSequence, renders_folder + "//0000.png",false,"/C");
    }
}