using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using UnityEditor;
using UnityEngine;
using UnityEngine.Rendering;
#if UNITY_PIPELINE_HDRP
using UnityEngine.Rendering.HighDefinition;
#endif
using UnityEngine.SceneManagement;

public class OCSceneControl : MonoBehaviour
{
    public enum LightingType // your custom enumeration
    {
        From3dsMAX,
        Exterior,
        Interior
    };

    [SerializeField] public  LightingType SceneLighting = LightingType.From3dsMAX;
    [Range(0, 2)][SerializeField] public float ReflectionProbesMult = 0.5f;
    [Range(0, 2)][SerializeField] public float DirectionalLightsMult = 0.5f;
    [Range(0, 2)][SerializeField] public float AreaLightsMult = 0.5f;
    [Range(0, 2)][SerializeField] public float SpotLightsMult = 0.5f;
    [Range(0, 2)][SerializeField] public float PointLightsMult = 0.5f;
    [Range(0, 2)][SerializeField] public float SelfIlluminationsPower = 0.66f;
    //public bool SkyMultOrExposure;
    [Range(0, 2)][SerializeField] public float EnvironmentMult = 0.5f;
    [Range(-10, 10)][SerializeField] public float Exposure = 0f;
    [Range(-100, 100)][SerializeField] public float Contrast = 0;
    [Range(-100, 100)][SerializeField] public float Saturation = 0;
    [Range(-100, 100)][SerializeField] public float WhiteBalance = 0f;
    public List<OCVrayMtl> selfillummats = new List<OCVrayMtl>();

    private OCLightingManager _lightManager;
    public OCLightingManager LightManager
    {
        get
        {
            if (_lightManager == null)
            {
                _lightManager = FindObjectOfType<OCLightingManager>();
            }
            return _lightManager;
        }
    }

    private OCVolumeManager _volumeManager;
    public OCVolumeManager volumeManager
    {
        get
        {
            if (_volumeManager == null)
            {
                _volumeManager = FindObjectOfType<OCVolumeManager>();
            }
            return _volumeManager;
        }
    }
    private OneClickSceneManager _sceneManager;
    public OneClickSceneManager sceneManager
    {
        get
        {
            if (_sceneManager == null)
            {
                _sceneManager = FindObjectOfType<OneClickSceneManager>();
            }
            return _sceneManager;
        }
    }
#if UNITY_EDITOR

    private void OnValidate()
    {
        
        ChangeLighting(SceneLighting);
        ChangeMults();
    }
#endif
    private OCPrefab _sunSystem;
    public OCPrefab SunSystem
    {
        get
        {
            if (_sunSystem == null)
            {
#if UNITY_EDITOR
                if (FindObjectOfType<OCPrefab>() != null)
                {
                    _sunSystem = FindObjectOfType<OCPrefab>();
                    return _sunSystem;
                }
                _sunSystem = GameObject.Instantiate(AssetDatabase.LoadAssetAtPath<GameObject>("Assets\\OneClickImport\\Prefabs\\SunSystem.prefab")).GetComponent<OCPrefab>();
                _sunSystem.name = "SunSystem";
#endif
            }
            return _sunSystem;   
        }
    }
    
    public void ChangeLighting(LightingType sl)
    {

        switch (sl)
        {
            case LightingType.From3dsMAX:
                //SceneManager.UnloadSceneAsync(files[0]);
                for (int i = 0; i < SunSystem.transform.childCount; i++)
                {
                    SunSystem.transform.GetChild(i).gameObject.SetActive(false);
                }
                break;
            case LightingType.Exterior:
                for (int i = 0; i < SunSystem.transform.childCount; i++)
                {
                    SunSystem.transform.GetChild(i).gameObject.SetActive(true);
                }
                SunSystem.ChangeHDRi("");

                break;
            case LightingType.Interior:
                for (int i = 0; i < SunSystem.transform.childCount; i++)
                {
                    SunSystem.transform.GetChild(i).gameObject.SetActive(true);
                }
                SunSystem.ChangeHDRi("Assets\\OneClickImport\\HDRi\\provence_studio_1k.HDR");


                break;
            default:
                break;
        }

        
    }
    public void AddSceneToBuildSettings(string sceneName)
    {
        //string MainFolder = "Assets/Application/Scenes/"; 

        

        //EditorBuildSettingsScene[] original = EditorBuildSettings.scenes;
        //EditorBuildSettingsScene[] newSettings = new EditorBuildSettingsScene[original.Length + 1];
        //System.Array.Copy(original, newSettings, original.Length);

        //Debug.Log("Original Build Lenght = " + original.Length);
        //Debug.Log("New Build Lenght = " + newSettings.Length);


        //int i = 0;
        //int index = original.Length;

      
        //    EditorBuildSettingsScene sceneToAdd = new EditorBuildSettingsScene( sceneName , true);
        //    newSettings[index] = sceneToAdd;
         
        //EditorBuildSettings.scenes = newSettings;

    }
    public void ChangeMults()
    {
       

        sceneManager.ReflectionProbesMult = ReflectionProbesMult;
        sceneManager.AreaLightsMult = AreaLightsMult;
        sceneManager.SpotLightsMult = SpotLightsMult;
        sceneManager.DirectionalLightsMult = DirectionalLightsMult;
        sceneManager.SelfIlluminationsPower = SelfIlluminationsPower;
#if UNITY_PIPELINE_HDRP

        sceneManager.EnvironmentMult= EnvironmentMult;
#endif
#if UNITY_PIPELINE_URP
      
#endif
#if UNITY_EDITOR
        if(LightManager!=null)
        LightManager.ChangeMults(CalculateY(640, AreaLightsMult), CalculateY(640, SpotLightsMult), CalculateY(640, DirectionalLightsMult), CalculateY(640, PointLightsMult));

        OCVolumeManager volumeManager = FindObjectOfType<OCVolumeManager>();
        try
        {
            volumeManager.ChangeExposure(Exposure);
            volumeManager.ChangeContrast(Contrast);
            volumeManager.ChangeSaturation(Saturation);
            volumeManager.ChangeWhiteBalance(WhiteBalance);
        }
        catch { Logger.Log(" Volume must be ready to change this on start!"); };


        if (volumeManager != null)
        {
            volumeManager.ChangeRefProbes(CalculateY(240, ReflectionProbesMult));
            volumeManager.changeEnvironment(CalculateY(240, EnvironmentMult) - 1);
        }
            ChangeEmissiveMaterialsMult();
#endif
    }
    public void ChangeEmissiveMaterialsMult( )
    {
        foreach (var item in selfillummats)
            if(item != null|| item.material!=null)
        {
           //  item.material.SetColor("_EmissionColor", 0.33f * item.selfIllumination_multiplier * item.Diffuse* CalculateY(640, SelfIlluminationsPower));

        }
        
    }
    float CalculateY( float n,float x)
    { 
        float y = (float)(1.6f * (n * Math.Pow(x, 10)));
        return y;
    }

    
    public Texture2D RenderTextureToTexture2D(Camera targetCamera)
    {
        //      
        RenderTexture renderTexture = new RenderTexture(128,128, 32,RenderTextureFormat.ARGBFloat,0);
        targetCamera.targetTexture = renderTexture;
        targetCamera.allowHDR = true;

        targetCamera.Render(); 
        //       ,   renderTexture
        Texture2D texture = new Texture2D(128, 128, TextureFormat.RGBAFloat, false);
        RenderTexture.active = renderTexture;
        texture.ReadPixels(new Rect(0, 0, 128, 128), 0, 0);
        texture.Apply();
        //byte[] bytes = texture.EncodeToEXR(Texture2D.EXRFlags.None);
        //System.IO.File.WriteAllBytes(Application.dataPath  + "\\reflectance.exr", bytes);
        //     
        targetCamera.targetTexture = null;
        RenderTexture.active = null;
        DestroyImmediate(renderTexture);

        return texture;
    }
    public void ControlExposureOfLights()
    {
        ChangeMults();


        if (AllCompute() )
        {
            //ChangeMults(0, 0, 0, 0, 0,0,0.5f);
            //float c = 0;
            //while ((c+= 0.1f )<=2 )
            //{
            //    ChangeMults(c , 0, 0, 0, 0, 0, 0);

            //    if(AllCompute())
            //    {
            //        ChangeMults(c-=0.1f, 0, 0, 0, 0, 0, 0);
            //        break;
            //    }
            //}
            //c = 0;
            //while ((c+= 0.1f )<=2 )
            //{
            //    ChangeMults(ReflectionProbesMult, c, 0, 0, 0, 0, 0);

            //    if(AllCompute())
            //    {
            //        ChangeMults(ReflectionProbesMult, c, 0, 0, 0, 0, 0);
            //        break;
            //    }
            //}
        }

    }
    bool AllCompute()
    {
#if UNITY_EDITOR
     //   return GetBrightestPixelColor(RenderTextureToTexture2D(FindObjectOfType<OCCamerasManager>().transform.GetChild(0).GetComponent<Camera>()));
#endif
        return false;
    }
    public bool GetBrightestPixelColor(Texture2D texture)
    {
        Color[] pixels = texture.GetPixels();
         
        float brightestIntensity =   0.97f;

        int sum = 0;
        foreach (Color pixel in pixels)
        {
            float intensity = pixel.grayscale;

            if (intensity > brightestIntensity)
            {
                sum++;   
                }
        }
        Debug.Log("OverBrightest Pixels Count is :" + sum);
        if (sum > 8192)
            return true;
        else return false;
    }
    public void ChangeMults(float _ReflectionProbesMult, float _DirectionalLightsMult, float _AreaLightsMult, float _SpotLightsMult, float _SelfIlluminationsPower, float _PointLightsMult, float _EnvironmentMult)
    {
        sceneManager.DirectionalLightsMult = DirectionalLightsMult= _DirectionalLightsMult;
        sceneManager.ReflectionProbesMult = ReflectionProbesMult = _ReflectionProbesMult;
        sceneManager.AreaLightsMult = AreaLightsMult = _AreaLightsMult;
        sceneManager.SpotLightsMult = SpotLightsMult = _SpotLightsMult;
        sceneManager.SelfIlluminationsPower = SelfIlluminationsPower = _SelfIlluminationsPower;
        sceneManager.PointLightsMult = PointLightsMult = _PointLightsMult;
        sceneManager.EnvironmentMult = EnvironmentMult = _EnvironmentMult;
        ChangeMults();
    }
    public void ResetValues()
    {
#if UNITY_PIPELINE_HDRP
       
        ChangeMults(0.5f, 0.5f, 0.5f, 0.5f, 0.66f, 0.5f, 0.5f);

#endif
        ChangeMults(0.5f, 0.5f, 0.5f, 0.5f, 0.33f, 0.5f, 0.5f);
        OCVolumeManager volumeManager = FindObjectOfType<OCVolumeManager>();
        volumeManager.ChangeExposure(Exposure = 0);
        volumeManager.ChangeContrast(Contrast = 0);
        volumeManager.ChangeSaturation(Saturation = 0);
        volumeManager.ChangeWhiteBalance(WhiteBalance = 0);
    }
    public void AddSelfIllumMatsToSerialize(List <OCVrayMtl> mats)
    {
        foreach (var item in mats)
        {
            selfillummats.Add(item);
        }
    }
    
     
}
