﻿using System.Collections;
using System.Collections.Generic; 
using System.IO;
using System.Linq;
using UnityEditor; 
using UnityEngine;
using UnityEngine.Rendering;
using UnityEngine.UI;
#if UNITY_EDITOR
[ExecuteInEditMode]
#endif
public class OCUIControl : MonoBehaviour
{
    [SerializeField] public CanvasGroup CG;
    [SerializeField] public Image VrayLogo;
    [SerializeField] public Image CoronaLogo;
    [SerializeField] public Text HDRPimg;
    [SerializeField] public Text URPimg;
    [SerializeField] public Text _3dsMaximg; 
    [SerializeField] public Slider ProgressBar; 
    public Image camImg;

    [SerializeField] public List<Sprite> Sprites = new List<Sprite>();
    private OneClickSceneManager _OCSM;
    public OneClickSceneManager OCSM
    {
        get
        {
            if (_OCSM == null)
                _OCSM = FindObjectOfType<OneClickSceneManager>();
            return _OCSM;
        }
    }
    void Awake()
    {
        if (Application.isPlaying)
        {
            SetUnityLogos();

#if UNITY_PIPELINE_HDRP
            
#endif
#if UNITY_PIPELINE_URP

#endif
            Button.alpha = 1;
            Sliders.alpha = 0;
        }else
        {
            TurnUI(false);
        }
    }
    public void LoadCamRenders()
    {
#if UNITY_EDITOR
        Sprites = new List<Sprite>();
        ConvertTextureToSpriteAndAddToList();
#endif
    }
    public List<string> camNames;
      

    private void ConvertTextureToSpriteAndAddToList()
    {
#if UNITY_EDITOR
        camNames = new List<string>();
        OCCamerasManager camerasManager = FindObjectOfType<OCCamerasManager>();
        if (camerasManager != null)
        {
            for (int i = 0; i < camerasManager.transform.childCount; i++)
            {
                string camName = camerasManager.transform.GetChild(i).name;
                camNames.Add(camName+"_max"); // Äîáàâëåíèå èìåíè êàìåðû áåç ñóôôèêñà "_max.png"
            }
        }

        string fullPath = Path.Combine(FindObjectOfType<OneClickSceneManager>().Path_Renders);
        if (!Directory.Exists(fullPath))
        {
            Debug.LogError("Directory does not exist: " + fullPath);
            return;
        }

        string[] fileEntries = Directory.GetFiles(fullPath, "*.png", SearchOption.AllDirectories);
        foreach (string filePath in fileEntries)
        {
              string fileName = Path.GetFileNameWithoutExtension(filePath);

            // Ïðîâåðêà, ñîîòâåòñòâóåò ëè èìÿ ôàéëà îäíîìó èç èìåí â camNames
            if (!camNames.Contains(fileName))
            {
                continue; // Ïðîïóñòèòü ôàéë, åñëè èìÿ íå ñîâïàäàåò
            }

            TextureImporter textureImporter = AssetImporter.GetAtPath(filePath) as TextureImporter;
            if (textureImporter != null)
            {
                textureImporter.textureType = TextureImporterType.Sprite;
                textureImporter.sRGBTexture = false;
                textureImporter.alphaSource = TextureImporterAlphaSource.None;
                
                AssetDatabase.ImportAsset(filePath, ImportAssetOptions.ForceUpdate);

                Sprite sprite = AssetDatabase.LoadAssetAtPath<Sprite>(filePath);
                if (sprite != null)
                {
                    DisableSpriteAlpha(sprite);
                    Sprites.Add(sprite); // Äîáàâëåíèå ñïðàéòà â ñïèñîê
                }

                Logger.Log("Converted to sprite and added to list: " + filePath);
            }
        }

        Logger.Log("Camera Renders Loaded: " + Sprites.Count);
#endif
    }

    public void OnChangeReflectionProbesMult(float value)
    {

        OSS.ChangeMults(value, OSS.DirectionalLightsMult, OSS.AreaLightsMult, OSS.SpotLightsMult, OSS.SelfIlluminationsPower, OSS.PointLightsMult, OSS.EnvironmentMult);
    }
    public void OnChangeDirectionalLightsMult(float value)
    {
        OSS.ChangeMults(OSS.ReflectionProbesMult, value, OSS.AreaLightsMult, OSS.SpotLightsMult, OSS.SelfIlluminationsPower, OSS.PointLightsMult, OSS.EnvironmentMult);

    }
    public void OnChangeAreaLightsMult(float value)
    {
        OSS.ChangeMults(OSS.ReflectionProbesMult, OSS.DirectionalLightsMult, value, OSS.SpotLightsMult, OSS.SelfIlluminationsPower, OSS.PointLightsMult, OSS.EnvironmentMult);

    }
     public void OnChangeSpotLightsMult(float value)
    {
        OSS.ChangeMults(OSS.ReflectionProbesMult, OSS.DirectionalLightsMult, OSS.AreaLightsMult, value, OSS.SelfIlluminationsPower, OSS.PointLightsMult, OSS.EnvironmentMult);

    }   public void OnChangeSelfIlluminationsPower(float value)
    {
        OSS.ChangeMults(OSS.ReflectionProbesMult, OSS.DirectionalLightsMult, OSS.AreaLightsMult, OSS.SpotLightsMult,value, OSS.PointLightsMult, OSS.EnvironmentMult);

    }
     public void OnChangePointLightsMult(float value)
    {
        OSS.ChangeMults(OSS.ReflectionProbesMult, OSS.DirectionalLightsMult, OSS.AreaLightsMult, OSS.SpotLightsMult, OSS.SelfIlluminationsPower, value, OSS.EnvironmentMult);

    }

    private OCSceneControl _OSS;

    OCSceneControl OSS
    {
        
        get
        {
            if (_OSS == null) 
            {
                _OSS = FindObjectOfType<OCSceneControl>();
            }
            return _OSS;
        }
    }
    public void OnChangeEnvironment(float value)
    {
        OSS.ChangeMults(OSS.ReflectionProbesMult, OSS.DirectionalLightsMult, OSS.AreaLightsMult, OSS.SpotLightsMult, OSS.SelfIlluminationsPower, OSS.PointLightsMult, value);
    }
    [SerializeField] CanvasGroup Button;
    [SerializeField] CanvasGroup Sliders;
    public void TurnUI(bool value )
    {
        Show3dsMAX(false);
        ABimage.enabled = false;
        if (value == false)
        {
            Button.alpha = 0;
            Sliders.alpha = 0;
        }
        else
        {
            Button.alpha = 1;
            Sliders.alpha = 1;
        }


    }
    public void SetUnityLogos()
    {
#if UNITY_PIPELINE_HDRP
        HDRPimg.text = "HDRP" + Application.unityVersion + " v." + Logger.Version;
          URPimg.transform.parent.GetComponent<Image>().enabled = false;
        URPimg.text="";
#endif
#if UNITY_PIPELINE_URP
        URPimg.text = "URP" + Application.unityVersion + " v." + Logger.Version;
        HDRPimg.transform.parent.GetComponent<Image>().enabled = false;
        HDRPimg.text = "";

#endif
        

    }
    public void Show3dsMAX(bool isShow)
    {
        if (OCSM.maxversion == "24.0") _3dsMaximg.text = "MAX 22";
        if (OCSM.maxversion == "25.2") _3dsMaximg.text = "MAX 23";
        if (isShow)
        {
            foreach (var item in _3dsMaximg.transform.parent.parent.GetComponentsInChildren<Image>())
            {
                item.color = new UnityEngine.Color(1, 1, 1, 1);
            }
            
        }
        else
        {
            foreach (var item in _3dsMaximg.transform.parent.parent.GetComponentsInChildren<Image>())
            {
                item.color = new UnityEngine.Color(1, 1, 1, 0);
            }
            _3dsMaximg.text = "";
        }
    }   public void ShowABImageVS(bool isShow)
    { 
        if (isShow)
        {
            URPimg.text= "URP" + Application.unityVersion + " v." + Logger.Version;
            URPimg.transform.parent.GetComponent<Image>().color = new UnityEngine.Color(1, 1, 1, 1);
        }
        else
        {
            URPimg.transform.parent.GetComponent<Image>().color = new UnityEngine.Color(1, 1, 1, 0);
            URPimg.text = "";
        }
    }
    public void ShowHideSliders()
    { 
            if (Sliders.alpha == 0) Sliders.alpha = 1; else Sliders.alpha = 0; 
    }
    void OnGUI()
    {
        Event e = new Event();
        while (Event.PopEvent(e))
        {

            if (e.rawType == EventType.MouseDown && e.button == 0 || e.rawType == EventType.MouseDrag)
            {

                //Ray ray = Camera.main.ScreenPointToRay(new Vector2(e.mousePosition.x, Screen.height - e.mousePosition.y));
                //RaycastHit hit;
                //if (Physics.Raycast(ray, out hit))
                //{
               // Debug.Log("Change AB Image");
                //Selection.activeGameObject = hit.collider.gameObject;
                //Color C = Color.red;
                //C.a = 0.33f;
                //Gizmos.color = C;
                ABimage.enabled = true;
                CG.alpha= 1;
                    ABimage.sprite = CurrentCameraSprite;
                    ABimage.fillAmount =( e.mousePosition.x / Screen.width);

                    //Debug.Log("sss");
                    // Gizmos.DrawCube(Selection.activeGameObject.transform.position, Selection.activeGameObject.GetComponent<MeshRenderer>().bounds.size);
                    //Gizmos.DrawMesh(hit.collider.gameObject.GetComponent<MeshFilter>().sharedMesh, hit.collider.gameObject.transform.position, hit.collider.gameObject.transform.rotation, hit.collider.gameObject.transform.localScale);
                //}
            }    if (e.rawType == EventType.MouseDown && e.button == 1 || e.rawType == EventType.MouseDrag)
            {
               // Debug.Log("Render: "+GetPixelColor(ABimage.sprite, e.mousePosition));
                
              
            }
        }
    }
    public void SetLogos(bool isVray )
    {
        if (isVray) {
            VrayLogo.enabled = true;
            CoronaLogo.enabled = false;
        }
        else
        {
            VrayLogo.enabled = false;
            CoronaLogo.enabled = true;

        }

    }
    #region AB_Testing
   [SerializeField] public Image ABimage;

    public void ChangeSprite(string n)
    {
        string assetPath = Path.Combine(OCSM.Path_Renders, n + "_max.png");
        Texture2D texture = LoadPNGTexture(assetPath);

        if (texture != null)
        {
            // Ñîçäàåì íîâûé Sprite èç çàãðóæåííîé òåêñòóðû
            Sprite sprite = Sprite.Create(texture, new Rect(0, 0, texture.width, texture.height), Vector2.one * 0.5f);

            // Òåïåðü ó âàñ åñòü Sprite, êîòîðûé ìîæíî èñïîëüçîâàòü
            ABimage.sprite = sprite;
            _CurrentCameraSprite = sprite;
        }
    }
    private Sprite _CurrentCameraSprite;
    public Sprite CurrentCameraSprite
    {
        get
        {
            
                if (Camera.main == null) return _CurrentCameraSprite;
                string assetPath = Path.Combine(OCSM.Path_Renders, Camera.main.name+ "_max.png");
                Texture2D texture = LoadPNGTexture(assetPath);

                if (texture != null)
                {
                    // Ñîçäàåì íîâûé Sprite èç çàãðóæåííîé òåêñòóðû
                    Sprite sprite = Sprite.Create(texture, new Rect(0, 0, texture.width, texture.height), Vector2.one * 0.5f);

                    // Òåïåðü ó âàñ åñòü Sprite, êîòîðûé ìîæíî èñïîëüçîâàòü
                    ABimage.sprite = sprite;
                    _CurrentCameraSprite = sprite;
                }
                 
            return _CurrentCameraSprite;
        }
    }
    private Texture2D LoadPNGTexture(string filePath)
    {
        if (File.Exists(filePath))
        {
            byte[] fileData = File.ReadAllBytes(filePath);
            Texture2D texture = new Texture2D(2, 2); // Ìèíèìàëüíûå ðàçìåðû, îíè áóäóò ïåðåçàïèñàíû
            if (texture.LoadImage(fileData))
            {
                return texture;
            }
            else
            {
                Debug.LogError("Failed to load image data from " + filePath);
                ABimage.enabled = false;
                ABimage.gameObject.SetActive(false);
            }
        }
        else
        {
            ABimage.enabled = false;
            ABimage.gameObject.SetActive(false);
            Logger.Log("File not found: " + filePath);
        }

        return null;
    }


    #endregion
    public static UnityEngine.Color GetPixelColor(Sprite sprite, Vector2 pos)
    {
        // Проверяем, чтобы спрайт был не нулевым
        if (sprite == null)
        {
            //Debug.LogError("Sprite is null.");
            return Color.clear; // Возвращаем пустой цвет в случае ошибки
        }

        // Получаем текстуру спрайта
        Texture2D texture = sprite.texture as Texture2D;

        // Проверяем, чтобы текстура была не нулевой
        if (texture == null)
        {
            Debug.LogError("Sprite does not have a valid texture.");
            return Color.clear; // Возвращаем пустой цвет в случае ошибки
        }

        // Переводим координаты в целочисленные значения
        int x = Mathf.FloorToInt(pos.x );
        int y = Mathf.FloorToInt(pos.y );

        // Проверяем, чтобы координаты находились в пределах текстуры
        if (x < 0 || x >= texture.width || y < 0 || y >= texture.height)
        {
            Debug.Log("Position is out of bounds.");
            return Color.clear; // Возвращаем пустой цвет в случае ошибки
        }

        // Получаем цвет пикселя
        Color pixelColor = texture.GetPixel(x, y);

        return pixelColor;
    }
    public Sprite DisableSpriteAlpha(Sprite sprite)
    {
        Texture2D texture = sprite.texture;
        Color[] pixels = texture.GetPixels();

        for (int i = 0; i < pixels.Length; i++)
        {
            pixels[i].a = 0; // Установка альфа-канала каждого пикселя в 0
        }

        Texture2D newTexture = new Texture2D(texture.width, texture.height);
        newTexture.SetPixels(pixels);
        newTexture.Apply();

        return Sprite.Create(newTexture, sprite.rect, sprite.pivot);
    }

}
