﻿#if UNITY_PIPELINE_HDRP
#warning UNITY_PIPELINE_HDRP is defined
#else
#warning UNITY_PIPELINE_HDRP is NOT defined
#endif
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
#if UNITY_EDITOR
using UnityEditor;
#endif
using UnityEngine;
using UnityEngine.Events;
using UnityEngine.Rendering;
using System;
using System.Reflection;
using UnityEngine.Experimental.GlobalIllumination;
using System.Data.Common;

using Object = UnityEngine.Object;
using System.Globalization;

#if UNITY_PIPELINE_HDRP
using UnityEngine.Rendering.HighDefinition;
#endif
#if UNITY_PIPELINE_URP
using UnityEngine.Rendering.Universal;
#endif
using System.IO;
#if UNITY_EDITOR
using UnityEditor.SceneManagement;
using UnityEngine.SceneManagement;
using Unity.VisualScripting;


#endif

[ExecuteInEditMode]
public class OCVolumeManager : MonoBehaviour
{
    [Range (2f,100)]
    [Header(" Rebake probes if Changed ")]
    [SerializeField] float StepSizeBetweenProbes = 8;
    [Range (0,10)]
    [SerializeField] float BlendDistance = 2;
   
    [Range (0.5f,500)]
    [SerializeField] float Size = 24;

    [SerializeField] Vector3 PositionShift = Vector3.zero; 
    
    public static OCVolumeManager instance; 
    private OneClickSceneManager _ocsm;

    public OneClickSceneManager ocsm
    {
        get
        {
            if (_ocsm == null)
            {
                _ocsm = FindObjectOfType<OneClickSceneManager>();
            }
            return _ocsm;
        }
    }
    List<ReflectionProbe> RPS = new List<ReflectionProbe>();

    public void ChangeRefProbes(float power)
    {
        if (transform.childCount == 0) return;
        {

        }
        if(RPS.Count==0)
        {
            RPS = transform.GetComponentsInChildren<ReflectionProbe>().ToList();
        }
        foreach (var item in RPS)
        {
            item.intensity = power;

#if UNITY_PIPELINE_HDRP

            HDAdditionalReflectionData data = item.GetComponent<HDAdditionalReflectionData>();
         if(data!=null)   data.multiplier = power;

#endif
        }
    }
  
    public Volume volume; 
    [ExecuteInEditMode]

    void Awake()
    {  
        transform.SetAsFirstSibling();
        CheckProfile();
        //if (instance == null)
        //{
        //    instance = this;
        //}
        //else
        //{
        //    DestroyImmediate(gameObject);
        //} 
        //name = "OCVolumeAndReflectionProbesManager";
       

        //newHashEdit = Size + BlendDistance + intensity + PositionShift.magnitude;
        //newhash = StepSizeBetweenProbes;

    }

    private Cubemap _envMap;
public Cubemap EnvMap
    {
        get
        {
            if(_envMap == null)
            {
#if UNITY_PIPELINE_HDRP
                if (!volume.profile.TryGet<HDRISky>(out var skyfog))
                {
                    skyfog = volume.profile.Add<HDRISky>(true);
                }

                skyfog.active = true;
                // skyfog.skyIntensityMode.value = SkyIntensityMode.Exposure;

                //  skyfog.skyIntensityMode.overrideState = true;
                //  skyfog.exposure = new FloatParameter(113);
                //   skyfog.exposure.overrideState = true;
                skyfog.exposure.value = ocsm.EnvironmentMult;

                if (skyfog.hdriSky != null)
                    _envMap = ((Cubemap)skyfog.hdriSky);
#endif
            }

            return _envMap;
        }
        set
        {
            _envMap= value;
#if UNITY_PIPELINE_HDRP

            if (!volume.profile.TryGet<HDRISky>(out var skyfog))
            {
                skyfog = volume.profile.Add<HDRISky>(false);
            }

            // skyfog.active = true;
            // skyfog.skyIntensityMode.value = SkyIntensityMode.Multiplier;

            //  skyfog.skyIntensityMode.overrideState = true;
          //  skyfog.exposure.value = sceneManager.EnvironmentMult;
          //   skyfog.multiplier = new MinFloatParameter(123, 0, true);
          //  skyfog.multiplier.overrideState = true;
          //   skyfog.exposure.overrideState = true;
            skyfog.hdriSky.Override(_envMap);
#endif
        }
    }
  
    public void changeEnvironment(float val )
    {
#if UNITY_PIPELINE_HDRP
        if (!volume.profile.TryGet<HDRISky>(out var skyfog))
        {
            skyfog = volume.profile.Add<HDRISky>(false);
        }
        //skyfog.active = false; 
        //if (typ)
        //{
        //    skyfog.skyIntensityMode.value = SkyIntensityMode.Exposure;
        //    skyfog.skyIntensityMode.overrideState = true;
            skyfog.exposure.value = val;
        //}
        //else
        //{
         //   skyfog.skyIntensityMode.value = SkyIntensityMode.Multiplier;
         //   skyfog.multiplier.overrideState = true;

        //    skyfog.multiplier = new MinFloatParameter(val, 0, true);

        //} 
        //skyfog.exposure.overrideState = true;
        // skyfog.multiplier.overrideState = true;

       // _envMap = ((Cubemap)skyfog.hdriSky);
       // skyfog.active = true;

#endif
    }
    [SerializeField] Volume vol => GetComponent<Volume>();
    public void CheckProfile()
    { 
        if (Application.isPlaying)
            if(volume == null)
        {
                volume = vol;
        }

        if (volume == null || volume?.profile.name != FindObjectOfType<OneClickSceneManager>().gameObject.name)
        {
            volume = gameObject.GetComponent<Volume>();
            if (volume == null) return;
            OneClickSceneManager ocsm = FindObjectOfType<OneClickSceneManager>();

            string vPath = ocsm.Path_Textures + "\\" + ocsm.gameObject.name;
 
#if UNITY_EDITOR
#if UNITY_PIPELINE_HDRP

            vPath += "_HDRP.asset";
#else
            vPath += "_URP.asset";
        
#endif

            VolumeProfile v = AssetDatabase.LoadAssetAtPath<VolumeProfile>(vPath);


            if (v != null)
                volume.profile = vol.profile = v;
#endif
#if UNITY_PIPELINE_HDRP


            try
            {

                FindObjectOfType<StaticLightingSky>().profile = volume.profile;
                FindObjectOfType<StaticLightingSky>().staticLightingSkyUniqueID = 1;
                FindObjectOfType<StaticLightingSky>().enabled = true;
            }
            catch { }
#endif

            if (EnvMap == null)
                EnvMap = ocsm.OcEnvMap;
        //    Debug.LogError("Suka");
        }
    }
    [ContextMenu("Create HDRP Volume Profile")]
    public void CreateHDRPVolumeProfile()
    {
        if (gameObject.GetComponent<Volume>() != null)
        {
            return; 
        }



        volume = gameObject.AddComponent<Volume>();

       
        string vPath = Path.Combine(ocsm.Path_Textures, ocsm.OCScene.OcName);
#if UNITY_EDITOR
#if UNITY_PIPELINE_HDRP

        vPath += "_HDRP.asset";
 
#if UNITY_2022_3_OR_NEWER

        AssetDatabase.CopyAsset("Assets/OneClickImport/VolumeProfiles/OCDefaultVolumeProfileHDRP_14+.asset", vPath);

#else
        AssetDatabase.CopyAsset("Assets/OneClickImport/VolumeProfiles/OCDefaultVolumeProfileHDRP.asset", vPath);
#endif
#endif
#if UNITY_PIPELINE_URP
        vPath += "_URP.asset";
        if (File.Exists(vPath))
        {
            File.Delete(vPath);
            Logger.Log($"Volume File Clear! {vPath} Try To Copy it!");

        }
        if (File.Exists(Path.GetFullPath("Assets/OneClickImport/VolumeProfiles/OCDefaultVolumeProfileURP.asset")))
        {
            Logger.Log(Path.GetFullPath("Assets/OneClickImport/VolumeProfiles/OCDefaultVolumeProfileURP.asset  File Existing!!! Try to Copy!"));
        } 
       if( AssetDatabase.CopyAsset("Assets/OneClickImport/VolumeProfiles/OCDefaultVolumeProfileURP.asset", OCTools.GetUnityPath( vPath))==true)
            Logger.Log($"Volume copyed!!! {vPath}");
       else
            Logger.Log($"Volume Not copyed!!! {vPath}");

        AssetDatabase.Refresh();
        if (!File.Exists(vPath))
        {
            Logger.Log($"Volume File didnt Created! {vPath}");
        }
        else
            Logger.Log($"Volume File Created! {vPath} Try To Load it!");
#endif

        VolumeProfile v = AssetDatabase.LoadAssetAtPath<VolumeProfile>(OCTools.GetUnityPath( vPath)) as VolumeProfile;
        volume.profile = v;
        volume.sharedProfile = v;

       EditorUtility.SetDirty(volume.profile);
       EditorUtility.SetDirty(volume.sharedProfile);
       EditorUtility.SetDirty(volume.gameObject);
       EditorUtility.SetDirty(volume);
        //EditorSceneManager.SaveScene(EditorSceneManager.GetActiveScene());
 
#endif
#if UNITY_PIPELINE_HDRP


        try
        {

            FindObjectOfType<StaticLightingSky>().profile = volume.profile;
            FindObjectOfType<StaticLightingSky>().staticLightingSkyUniqueID = 1;
            FindObjectOfType<StaticLightingSky>().enabled = true;
        }
        catch { } 
#endif
     
        if (EnvMap==null)
        EnvMap = ocsm.OcEnvMap;   
    }

    public void MakeReflectionProbesByCameras(float radious)
    {
        radious = Mathf.Clamp(radious, 10, 30);

        OCCamerasManager oCCameras = FindObjectOfType<OCCamerasManager>();
        for (int i = 0; i < oCCameras.transform.childCount; i++)
        //for (int i = 0; i < 1; i++)
        {
            Vector3 newPosition = oCCameras.transform.GetChild(i).transform.position + oCCameras.transform.GetChild(i).transform.forward * 2;
            newPosition.y = oCCameras.transform.GetChild(i).transform.position.y; // сохраняем ту же высоту

            AddReflectionProbe(newPosition, oCCameras.transform.GetChild(i).name + "_RefProbeByCamera", radious);
           // AddReflectionProbe(Vector3.zero, oCCameras.transform.GetChild(i).name + "_RefProbeByCamera", 25);
        }
        if (GameObject.FindObjectsOfType<Camera>().Length == 0) AddReflectionProbe(Vector3.zero, "_DefaultCenter", radious);
    }
    //public static void SetIndirectResolution(float val)
    //{
    //    SetFloat("m_LightmapEditorSettings.m_Resolution", val);
    //}

    //public static void SetAmbientOcclusion(float val)
    //{
    //    SetFloat("m_LightmapEditorSettings.m_CompAOExponent", val);
    //}

    //public static void SetBakedGiEnabled(bool enabled)
    //{
    //    SetBool("m_GISettings.m_EnableBakedLightmaps", enabled);
    //}

    //public static void SetFinalGatherEnabled(bool enabled)
    //{
    //    SetBool("m_LightmapEditorSettings.m_FinalGather", enabled);
    //}

    //public static void SetFinalGatherRayCount(int val)
    //{
    //    SetInt("m_LightmapEditorSettings.m_FinalGatherRayCount", val);
    //}

    //public static void SetFloat(string name, float val)
    //{
    //    ChangeProperty(name, property => property.floatValue = val);
    //}

    //public static void SetInt(string name, int val)
    //{
    //    ChangeProperty(name, property => property.intValue = val);
    //}

    //public static void SetBool(string name, bool val)
    //{
    //    ChangeProperty(name, property => property.boolValue = val);
    //}

    //public static void ChangeProperty(string name, Action<SerializedProperty> changer)
    //{
    //    var lightmapSettings = getLighmapSettings();
    //    var prop = lightmapSettings.FindProperty(name);
    //    if (prop != null)
    //    {
    //        changer(prop);
    //        lightmapSettings.ApplyModifiedProperties();
    //    }
    //    else Debug.LogError("lighmap property not found: " + name);
    //}

    //static SerializedObject getLighmapSettings()
    //{
    //    var getLightmapSettingsMethod = typeof(LightmapEditorSettings).GetMethod("GetLightmapSettings", BindingFlags.Static | BindingFlags.NonPublic);
    //    var lightmapSettings = getLightmapSettingsMethod.Invoke(null, null) as Object;
    //    return new SerializedObject(lightmapSettings);
    //}

    public void SetLightmapSettings()
    {
#if UNITY_EDITOR
        //SetBakedGiEnabled(true);
        //SetIndirectResolution(1.337f);
        //SetAmbientOcclusion(1.337f);
        //SetFinalGatherEnabled(true);
        //SetFinalGatherRayCount(1337);
  
        


        LightingSettings ls;
        EditorSceneManager.SetActiveScene(SceneManager.GetActiveScene());
        UnityEditor.Lightmapping.TryGetLightingSettings(out ls);
        if (ls == null)
        {
            UnityEditor.Lightmapping.lightingSettings=new LightingSettings();
        }
        ls.name = ocsm.OCScene.OcName;

       
        // get environment Lighting volume profile


        
        
        // Configure the LightingSettings object
           // ls.albedoBoost = 1.1f;
             ls.bakedGI = true;

             ls.lightmapper = LightingSettings.Lightmapper.ProgressiveGPU;
        //ls.directSampleCount = 256;
        //ls.indirectSampleCount = 512;
        //ls.environmentSampleCount = 256;
        //ls.lightmapResolution = 320;
        //ls.lightmapPadding = 2;
        //ls.lightmapMaxSize = 4096;
         
        //    ls.ao = true;
        //    ls.aoMaxDistance = 0.33f;
        //    ls.aoExponentDirect = 0.33f;
        //    ls.aoExponentIndirect = 0.33f;
      
#if UNITY_PIPELINE_URP
        UnityEngine.RenderSettings.fog = false;
#endif
        if (ls == null) Debug.LogError("ls is null");
            else UnityEditor.Lightmapping.lightingSettings = ls;
        string lsPath = Path.Combine(ocsm.Path_Scenes, ls.name + "_lightSettings.lighting");

        try
        {

            LightingSettings set = AssetDatabase.LoadAssetAtPath<LightingSettings>(lsPath);
            if (set == null) ;
               // AssetDatabase.CreateAsset(ls, lsPath);
        } catch (Exception e) { }
       
#endif
    }
    
    
    [ContextMenu("CreateReflectionProbes")]
    public void CreateReflectionProbes()
    {
        SetLightmapSettings();
        RPS.Clear();
        while(transform.childCount>0)
        {
            DestroyImmediate(transform.GetChild(0).gameObject);
        }


        Bounds bounds = GetLocalBoundsForObject(ocsm.gameObject);

        float maxB = bounds.extents.x;
        if (maxB < bounds.max.z) maxB = bounds.max.z;
        //StepSizeBetweenProbes = maxB / 10;
        //StepSizeBetweenProbes = Mathf.Clamp(StepSizeBetweenProbes, 0.5f, 100);

        int maxpp = 0;
        //for (float i = 1; i < bounds.max.x; i += StepSizeBetweenProbes)
        //    for (float j = 1; j < bounds.max.z; j += StepSizeBetweenProbes)
        //    {
        //        float x = Mathf.Lerp(bounds.min.x, bounds.max.x, i / bounds.max.x);
        //        float z = Mathf.Lerp(bounds.min.z, bounds.max.z, j / bounds.max.z);

        //        if (maxpp++ > 30) return;
        //        AddReflectionProbe(new Vector3(x, posy(), z) + PositionShift, (i + j).ToString());
        //    }
        MakeReflectionProbesByCameras(maxB);

    }

    float posy()
    {

        float res = 0;
#if UNITY_EDITOR

        OCCamerasManager ocmr =FindObjectOfType<OCCamerasManager>();
        if (ocmr != null)
        {
            if (ocmr.transform.childCount > 0)
                res = ocmr.transform.GetChild(0).position.y;
        }
#endif
        return res;
    }

    void EditProbes()
    {

        RPS=transform.GetComponentsInChildren<ReflectionProbe>().ToList();



        foreach (var item in RPS)
        {
            item.blendDistance = BlendDistance;
            item.size= new Vector3(Size, 30, Size);
            item.intensity = FindObjectOfType<OCSceneControl>().ReflectionProbesMult;


#if UNITY_PIPELINE_HDRP
            var probe = item.GetComponent<HDAdditionalReflectionData>();
            if (probe != null)
            {
                //probe.SetTexture(ProbeSettings.Mode.Custom, AssetDatabase.LoadAssetAtPath<Cubemap>(System.IO.Path.Combine(sceneManager.TexturesPath, "RefProbe_" + _name + ".exr")));
                probe.multiplier = item.intensity;
                probe.influenceVolume.boxSize = new Vector3( Size, 30, Size);
                probe.influenceVolume.boxBlendDistancePositive = new Vector3(BlendDistance, BlendDistance, BlendDistance);
                probe.influenceVolume.boxBlendDistanceNegative = new Vector3(BlendDistance, BlendDistance, BlendDistance);
            }
#endif
        }
        int s = 0;
        Bounds bounds = GetLocalBoundsForObject(ocsm.gameObject);
        for (float i = 1; i < bounds.max.x; i += StepSizeBetweenProbes)
            for (float j = 1; j < bounds.max.z; j += StepSizeBetweenProbes)
            {
                float x = Mathf.Lerp(bounds.min.x, bounds.max.x, i / bounds.max.x);
                float z = Mathf.Lerp(bounds.min.z, bounds.max.z, j / bounds.max.z);

                transform.GetChild(s).transform.position=  new Vector3(x, posy(), z) + PositionShift;
                s++;
                if (s > 30) return;
            }
    }
    void AddReflectionProbe(Vector3 pos, string _name,float radious)
    {
        Size = (int) StepSizeBetweenProbes * 3;
        BlendDistance = StepSizeBetweenProbes / 2;
        Bounds bounds = GetLocalBoundsForObject(ocsm.gameObject);
        ReflectionProbe RP = new GameObject().AddComponent<ReflectionProbe>();
        RP.transform.SetParent(transform);
        RP.transform.position = pos;
        RP.name = "ReflectionProbe_"+_name;
        RP.size = new Vector3(10, 10, 10);
        RP.intensity = 1* ocsm.ReflectionProbesMult;
        RP.blendDistance = BlendDistance;
        RP.timeSlicingMode = UnityEngine.Rendering.ReflectionProbeTimeSlicingMode.AllFacesAtOnce;
        RP.enabled = true;

        RP.mode = ReflectionProbeMode.Baked;

       // RP.refreshMode = UnityEngine.Rendering.ReflectionProbeRefreshMode.OnAwake;


        //Lightmapping.BakeReflectionProbe(RP, System.IO.Path.Combine(sceneManager.TexturesPath, "RefProbe_" + _name + ".exr"));
        //RP.customBakedTexture = AssetDatabase.LoadAssetAtPath<Cubemap>(System.IO.Path.Combine(sceneManager.TexturesPath, "RefProbe_" + _name + ".exr"));


#if UNITY_PIPELINE_HDRP
        
        RPS.Add(RP);

        var probe = RP.GetComponent<HDAdditionalReflectionData>();
        if (probe == null) return;
        //probe.SetTexture(ProbeSettings.Mode.Custom, AssetDatabase.LoadAssetAtPath<Cubemap>(System.IO.Path.Combine(sceneManager.TexturesPath, "RefProbe_" + _name + ".exr")));
        probe.multiplier = ocsm.ReflectionProbesMult;
        probe.influenceVolume.boxSize = new Vector3(Size, 30, Size);
         RP.mode = ReflectionProbeMode.Realtime;


        probe.influenceVolume.boxBlendDistancePositive = new Vector3(BlendDistance, BlendDistance, BlendDistance);
        probe.influenceVolume.boxBlendDistanceNegative = new Vector3(BlendDistance, BlendDistance, BlendDistance);
#endif
    }

#if UNITY_PIPELINE_HDRP
    public void BakeAllReflectionProbes()
    {
#if UNITY_EDITOR
        foreach (var RP in RPS)
        {


            UnityEditor.Lightmapping.BakeReflectionProbe(RP,System.IO.Path.Combine(  ocsm.Path_Textures,RP.name+".exr"));
            RP.customBakedTexture=AssetDatabase.LoadAssetAtPath<Cubemap>(System.IO.Path.Combine(ocsm.Path_Textures, RP.name + ".exr"));

            var probe = RP.GetComponent<HDAdditionalReflectionData>();
            probe.SetTexture(ProbeSettings.Mode.Custom, AssetDatabase.LoadAssetAtPath<Cubemap>(System.IO.Path.Combine(ocsm.Path_Textures, RP.name + ".exr")));
            probe.mode = ProbeSettings.Mode.Custom;
        }
#endif
    }
#endif
    public Bounds GetLocalBoundsForObject(GameObject go)
    {
        var referenceTransform = go.transform;
        var b = new Bounds(Vector3.zero, Vector3.zero);
        RecurseEncapsulate(referenceTransform, ref b);
        return b;

        void RecurseEncapsulate(Transform child, ref Bounds bounds)
        {
            var mesh = child.GetComponent<MeshFilter>();
            if (mesh)
            {
                if (mesh.sharedMesh == null) return;
                 
                var lsBounds = mesh.sharedMesh.bounds;
                var wsMin = child.TransformPoint(lsBounds.center - lsBounds.extents);
                var wsMax = child.TransformPoint(lsBounds.center + lsBounds.extents);
                bounds.Encapsulate(referenceTransform.InverseTransformPoint(wsMin));
                bounds.Encapsulate(referenceTransform.InverseTransformPoint(wsMax));
            }
            foreach (Transform grandChild in child.transform)
            {
                RecurseEncapsulate(grandChild, ref bounds);
            }
        }
    }
    private void OnDrawGizmosSelected()
    {
         
            var bounds = GetLocalBoundsForObject(ocsm.gameObject);
            Gizmos.color = Color.red;
            Gizmos.DrawWireCube(bounds.center, bounds.size);
        UnityEngine.Random.seed = 0;
        Gizmos.color = UnityEngine.Random.ColorHSV();
        Gizmos.color = new Color(Gizmos.color.r, Gizmos.color.g, Gizmos.color.b, 0.33f);
        for (int i = 0; i < transform.childCount; i++)
        {
            Gizmos.DrawCube(transform.GetChild(i).position, Vector3.one * Size); 
        }
    }
    public void ChangeExposure(float val)
    {
#if UNITY_PIPELINE_HDRP
        if (!volume.profile.TryGet<ColorAdjustments>(out var skyfog))
        {
            skyfog = volume.profile.Add<ColorAdjustments>(false);
        } 
        skyfog.postExposure.value = val;
#endif
#if UNITY_PIPELINE_URP
        try
        {
            volume = GetComponent<Volume>();
            if (!volume) throw new System.NullReferenceException(nameof(Volume));

            VolumeProfile volumeProfile; volumeProfile = GetComponent<Volume>()?.profile;
            ColorAdjustments CA;
            if (!volumeProfile) throw new System.NullReferenceException(nameof(VolumeProfile));
            if (!volumeProfile.TryGet(out CA)) throw new System.NullReferenceException(nameof(CA));

            CA.postExposure.value = val;
        }
        catch { };
#endif
    } public void ChangeContrast(float val)
    {
#if UNITY_PIPELINE_HDRP
        if (!volume.profile.TryGet<ColorAdjustments>(out var skyfog))
        {
            skyfog = volume.profile.Add<ColorAdjustments>(false);
        } 
        skyfog.contrast.value = val;
#endif
#if UNITY_PIPELINE_URP
        try
        {
            volume = GetComponent<Volume>();
            if (!volume) throw new System.NullReferenceException(nameof(Volume));

            VolumeProfile volumeProfile; volumeProfile = GetComponent<Volume>()?.profile;
            ColorAdjustments CA;
            if (!volumeProfile) throw new System.NullReferenceException(nameof(VolumeProfile));
            if (!volumeProfile.TryGet(out CA)) throw new System.NullReferenceException(nameof(CA));

            CA.contrast.value = val;
        }
        catch { Logger.Log("Error ChangeContrast(float val) "); }
#endif




    }
    public void ChangeSaturation(float val)
    {
#if UNITY_PIPELINE_HDRP
        if (!volume.profile.TryGet<ColorAdjustments>(out var skyfog))
        {
            skyfog = volume.profile.Add<ColorAdjustments>(false);
        } 
        skyfog.saturation.value = val;
#endif
#if UNITY_PIPELINE_URP

        volume = GetComponent<Volume>();
        if (!volume) throw new System.NullReferenceException(nameof(Volume));

        VolumeProfile volumeProfile; volumeProfile = GetComponent<Volume>()?.profile;
        
        ColorAdjustments CA;
        if (!volumeProfile) throw new System.NullReferenceException(nameof(VolumeProfile));
        if (!volumeProfile.TryGet(out CA)) throw new System.NullReferenceException(nameof(CA));

        CA.saturation.value = val;
#endif
    }
    public void ChangeWhiteBalance(float val)
    {
#if UNITY_PIPELINE_HDRP
        if (!volume.profile.TryGet<WhiteBalance>(out var skyfog))
        {
            skyfog = volume.profile.Add<WhiteBalance>(false);
        } 
        skyfog.temperature.value = val;

#endif
#if UNITY_PIPELINE_URP

        volume = GetComponent<Volume>();
        if (!volume) throw new System.NullReferenceException(nameof(Volume));

        VolumeProfile volumeProfile; volumeProfile = GetComponent<Volume>()?.profile;
        WhiteBalance CA;
        if (!volumeProfile) throw new System.NullReferenceException(nameof(VolumeProfile));
        if (!volumeProfile.TryGet(out CA)) throw new System.NullReferenceException(nameof(CA));

        CA.temperature.value = val;
#endif
    }
}
